/*
 * FastaHeaderParseStrings.java
 *
 * Created on Oct 8, 2007, 4:12:56 PM
 *
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package nl.uu.chem.bioinformatics.statquant.model;

import java.io.IOException;
import java.io.InputStream;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 *
 * @author Toorn101
 */
public class FastaHeaderRegexes {

    TreeMap<String, String> regexes = new TreeMap<String, String>();
    private static FastaHeaderRegexes instance;

    private FastaHeaderRegexes() {
        InputStream xmlstream = null;
        try {
            xmlstream = this.getClass().getResourceAsStream("/fasta_regexes.xml");

            DOMParser parser = new DOMParser();
//            parser.setFeature("http://xml.org/sax/features/validation", false);
//            parser.setErrorHandler(new org.xml.sax.ErrorHandler() {
//
//                @Override
//                public void warning(SAXParseException exception) throws SAXException {
//                    System.out.println(exception.toString());
//                }
//
//                @Override
//                public void error(SAXParseException exception) throws SAXException {
//                    System.out.println(exception.toString());
//                }
//
//                @Override
//                public void fatalError(SAXParseException exception) throws SAXException {
//                    System.out.println(exception.toString());
//                }
//            });
            parser.parse(new InputSource(xmlstream));

            Document doc = parser.getDocument();

            NodeList types = doc.getElementsByTagName("type");

            for (int count = 0; count < types.getLength(); count++) {
                String description = null;
                String regex = null;
                NodeList children = types.item(count).getChildNodes();
                for (int nodescount = 0; nodescount < children.getLength(); nodescount++) {
                    if (children.item(nodescount).getNodeName().equals("regex")) {
                        regex = children.item(nodescount).getFirstChild().getNodeValue();
                    }
                    if (children.item(nodescount).getNodeName().equals("description")) {
                        description = children.item(nodescount).getFirstChild().getNodeValue();
                    }
                }
                if (description != null && regex != null) regexes.put(description, regex);
            }
        } catch (SAXException ex) {
            Logger.getLogger(FastaHeaderRegexes.class.getName()).log(Level.SEVERE, null, ex);
            System.out.println(ex.toString());
        } catch (IOException ex) {
            Logger.getLogger(FastaHeaderRegexes.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                if (xmlstream != null) {
                    xmlstream.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(FastaHeaderRegexes.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        //TODO error handling
    }

    public static FastaHeaderRegexes getInstance() {
        if (instance == null) {
            instance = new FastaHeaderRegexes();
        }
        return instance;
    }

    /**
     * Get a list with all the descriptions as a string array
     * @return string array with descriptions, which double as map keys.
     */
    public String[] getDescriptions() {
        return regexes.keySet().toArray(new String[0]);
    }
    
    /**
     * Return the regex as a string from the description
     * @param description the exact description string as found in getDescriptions
     * @see getDescriptions()
     * @return the regex
     */
    public String getRegex(String description) {
        return regexes.get(description);
    }
}